@@ -88,7 +88,7 @@ impl<'a> Context<'a> {
 
 pub struct Lexer {
     debug: bool,
-    raw_string: bool,
+    use_origin_string: bool,
     tokens: Vec<Token>,
 }
 
@@ -111,13 +111,14 @@ impl<'a> Lexer {
     pub fn new() -> Self {
         Lexer {
             debug: false,
-            raw_string: false,
+            use_origin_string: false,
             tokens: Vec::<Token>::new(),
         }
     }
 
-    pub fn set_raw_string(&mut self, raw_string: bool) {
-        self.raw_string = raw_string;
+    // if use origin string, lexer won't escape special chars and keep the quotes or string boundaries.
+    pub fn set_use_origin_string(&mut self, use_origin_string: bool) {
+        self.use_origin_string = use_origin_string;
     }
 
     pub fn run(&mut self, input: &'a str) -> Result<Vec<Token>, LexError> {
@@ -273,19 +274,23 @@ impl<'a> Lexer {
     fn read_number(&mut self, ctx: &mut Context) -> LexResult {
         let mut expo = ('E', 'e');
         let mut num_str: Vec<u8> = Vec::new();
+        let mut hex = false;
         if self.check_current(ctx, '0') && self.check_next2(ctx, 'x', 'X') {
             expo = ('P', 'p');
             ctx.skip_into(2, &mut num_str);
+            hex = true;
         }
+        let is_digit = |c| {
+            (hex && Lexer::is_hex_digit(c)) || (!hex && Lexer::is_digit(c)) || (c as char) == '.'
+        };
         loop {
-            if self.check_next2(ctx, expo.0, expo.1) {
+            if self.check_current_if(ctx, is_digit) {
+                ctx.skip_into(1, &mut num_str)
+            } else if self.check_current2(ctx, expo.0, expo.1) {
                 ctx.skip_into(1, &mut num_str);
-                if self.check_next2(ctx, '-', '+') {
+                if self.check_current2(ctx, '-', '+') {
                     ctx.skip_into(1, &mut num_str)
                 }
-            }
-            if self.check_current_if(ctx, |c| Lexer::is_hex_digit(c) || (c as char) == '.') {
-                ctx.skip_into(1, &mut num_str)
             } else {
                 break;
             }
@@ -425,13 +430,16 @@ impl<'a> Lexer {
     }
 
     fn read_short_string(&mut self, ctx: &mut Context) -> LexResult {
-        let start = ctx.get();
-        ctx.next();
         let mut bytes: Vec<u8> = Vec::new();
+        let start = ctx.get();
+        if self.use_origin_string {
+            bytes.push(start.unwrap());
+        }
+        ctx.next();
         let unfinished_error: &'static str = "unfinished string";
         while ctx.get() != start {
             match ctx.get() {
-                Some(b'\\') if self.raw_string => ctx.skip_into(2, &mut bytes),
+                Some(b'\\') if self.use_origin_string => ctx.skip_into(2, &mut bytes),
                 Some(b'\\') => self.try_read_esc(ctx, &mut bytes)?,
                 Some(c) => {
                     if Lexer::is_line_break(c) {
@@ -444,6 +452,9 @@ impl<'a> Lexer {
                 None => return lex_error!(self, ctx, unfinished_error),
             }
         }
+        if self.use_origin_string {
+            bytes.push(ctx.get().unwrap());
+        }
         if let Ok(string) = String::from_utf8(bytes) {
             ctx.next();
             success!((TokenType::String, TokenValue::Str(string)))
@@ -480,22 +491,36 @@ impl<'a> Lexer {
         sem: &str,
     ) -> Result<Source, LexError> {
         let line = ctx.line;
+        let mut start = 0;
+
+        if self.use_origin_string {
+            start = ctx.current - 2 - sep_count;
+        }
 
         // skip first line break
         if self.check_current_if(ctx, |c| Lexer::is_line_break(c)) {
             self.read_line_break(ctx)?;
         }
 
-        let start = ctx.current;
+        if !self.use_origin_string {
+            start = ctx.current;
+        }
+
         let col = ctx.col;
         while let Some(c) = ctx.get() {
             match c {
                 b']' => {
                     if self.try_read_long_string_boundary(ctx, b']') == sep_count as i8 {
+                        let length;
+                        if self.use_origin_string {
+                            length = ctx.current - start;
+                        } else {
+                            length = ctx.current - 2 - sep_count - start;
+                        }
                         return Ok(Source {
                             line,
                             pos: start,
-                            length: ctx.current - 2 - sep_count - start,
+                            length,
                             col,
                         });
                     } else {
@@ -789,7 +814,7 @@ impl<'a> Lexer {
         }
     }
 
-    fn check_if(&self, src: Option<u8>, f: fn(u8) -> bool) -> bool {
+    fn check_if(&self, src: Option<u8>, f: impl Fn(u8) -> bool) -> bool {
         match src {
             Some(c) => f(c),
             None => false,
@@ -800,7 +825,11 @@ impl<'a> Lexer {
         self.check(ctx.get(), c)
     }
 
-    fn check_current_if(&self, ctx: &Context, f: fn(u8) -> bool) -> bool {
+    fn check_current2(&self, ctx: &Context, c1: char, c2: char) -> bool {
+        self.check(ctx.get(), c1) || self.check(ctx.get(), c2)
+    }
+
+    fn check_current_if(&self, ctx: &Context, f: impl Fn(u8) -> bool) -> bool {
         self.check_if(ctx.get(), f)
     }
 
